#
# Pythoned File Manager
#
#  Eugene Popov aka EugeneP, 2012
#    mailto:popov_e_n@mail.ru
#    http://uwolke.ru
#    http://wiz.rusbase.net
#
import pygame

try:
##	raise ImportError
	import threading as _threading
except ImportError:
	import dummy_threading as _threading

import Config

class RenderThread( _threading.Thread ):
	def __init__(self, compositor):
		self.__comp  = compositor
		self.__clock = pygame.time.Clock()
		_threading.Thread.__init__(self)
		self.setDaemon(True)

	def run(self):
		while self.isAlive():
			try:
				self.__clock.tick(30)
				self.__comp.frame_flip()
			except:
				return



class Compositor:
	def __init__( self, screen ):
		self.__layers = {}
		self.__screen = screen
		self.__order = []
		self.__lock  = _threading.Lock()

	def new_layer( self, name, x, y, width, height, bgsurface=None):
		l = CompositorLayer(width, height, bgsurface=None).set_xy( x, y )
		self.__layers[name] = l
		self.__order.insert( 0, name )
		return l

	def get_layer( self, name ):
		return self.__layers[name]

	def frame_flip( self, force=False ):
		self.__lock.acquire()

		vchanges = reduce( lambda v,x : v or ( x.get_visible() and x.is_changed() ), self.__layers.values(), False )
		if not vchanges and not force:
			self.__lock.release()
			return

		self.__screen.blit( Config.p('_BGM'), (0,0) )
		for k in self.__order:
			L = self.__layers[k]
			if L.get_visible():
				L.clear_changed()
				self.__screen.blit(L.surface(), dest=L.get_xy(), special_flags = L.get_blit_flags())
		pygame.display.flip()
		self.__lock.release()


class CompositorLayer:
	""" Visible layer """
	def __init__( self, width, height, bgsurface=None ):
		self.__surf = pygame.Surface( (width, height) , flags=pygame.SRCALPHA, depth=32)
		self.__surf.fill( (0,0,0,0) )
		#self.__surf.set_colorkey( (0,0,0,0) )
		self.__bb   = pygame.Surface( (width, height), flags=pygame.SRCALPHA, depth=32)
		self.__bb.fill( (0,0,0,0) )
		#self.__bb.set_colorkey( (0,0,0,0) )
		self.__visible = True
		self.__opt = 0
		self.__changed = True
		self._x = 0
		self._y = 0

	def set_xy( self, x, y ):
		self._x = x
		self._y = y
		return self

	def get_xy(self):
		return ( self._x, self._y )

	def is_changed(self):
		return self.__changed

	def clear_changed(self):
		self.__changed = False
		return self

	def mark_changed(self):
		self.__changed = True

	def surface(self):
		return self.__surf

	def get_width(self):
		return self.__surf.get_width()

	def get_height(self):
		return self.__surf.get_height()

	def get_visible(self):
		return self.__visible

	def set_visible(self, val):
		self.__visible = val
		Config.p('_compositor').frame_flip(True)
		return self

	def clear(self):
		"""Start drawing"""
		self.__surf.fill( (0,0,0,0) )

	def start(self):
		"""Start drawing"""
		self.__bb.fill( (0,0,0,0) )
		return self.__bb

	def stop(self):
		"""Stop drawing"""
		self.__surf = self.__bb.copy()
		self.__changed = True
##		Config.p('_compositor').frame_flip()

	def blit(self, src, pos):
		"""Blit on layer"""
		self.__bb.blit( src, pos )

	def set_blit_flags(self, opt):
		self.__opt = opt
		return self

	def get_blit_flags(self):
		return self.__opt

