
## gopher client
##
##

import select, socket
import urlparse, StringIO

from pfm import ModLoad
Config		= ModLoad("core", "Config")
Menu		= ModLoad("core", "Menu")
Log			= ModLoad("core", "Log")
Informer	= ModLoad("core", "Informer")

gtypD = {
'0' : 'Plain text file',
'1' : 'Directory menu listing',
'2' : 'CSO search query',
'3' : 'Error message',
'4' : 'BinHex encoded text file',
'5' : 'Binary archive file',
'6' : 'UUEncoded text file',
'7' : 'Search engine query',
'8' : 'Telnet session pointer',
'9' : 'Binary file',
'g' : 'GIF image',
'h' : 'Hyperlink',
'i' : 'Informational message',
'I' : 'Image file',
's' : 'Audio file',
'T' : 'tn3270 session pointer',
# meet in whildlife
'p' : 'PNG image',
}

grp_type_downloadable = ['0','1','4','5','6','9','g','I','p','s',]
grp_type_viewastext   = ['0','1','6',]
grp_type_viewasimage  = ['g','I','p',]

def entry2url(gopherentry):
	if gopherentry['port']!=70:
		return 'gopher://'+gopherentry['host']+':'+gopherentry['port']+gopherentry['path']
	else:
		return 'gopher://'+gopherentry['host']+gopherentry['path']

def url2entry(url):
	res = urlparse.urlparse(url)
	if res.scheme != 'gopher':
		raise ValueError(url)
	entry = {
		'type': 'gopher',
		'text': url,
		'gtype': '1',
		'xtype': None,
		'path' : res.path,
		'port' : 70,
	}
	if ':' in res.netloc:
		entry['host'], entry['port'] = res.netloc.split(':',1)
		entry['port'] = int(entry['port'])
	else:
		entry['host'] = res.netloc

def download_data( dest_datafile, host, path='/', port=70, quiet=False ):
	Log.msg( 'gopher get: ',host,':',port, path )

	s = None

	def cn1_wait():
		cn1_wait.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
		cn1_wait.sock.setblocking(0)
		try:
			cn1_wait.sock.connect_ex((host, port))
		except socket.gaierror, e:
			cn1_wait.sock = None

	Menu.Wait( cn1_wait, "Connecting")
	s = cn1_wait.sock

	if s == None:
		return False

	def cn2_wait(sock=s):
		r,w,x = select.select( [sock], [sock], [sock], 0 )
		if len( w ):
			return None
		return "Connected"

	Menu.PfmLongtime( Config.p('_compositor').get_layer('dialog'), cn2_wait, quiet=quiet ).loop()

	s.sendall(path + "\r\n", socket.MSG_DONTWAIT)

	def dl_wait( sock=s, bs=10240, f = dest_datafile ):
		try:
			buf = s.recv( bs, socket.MSG_DONTWAIT )
		except socket.error, e:
			if e[0] != 11:
				Log.msg('Transfer terminated by error ', e[0])
				raise
			return "Wait"
		f.write(buf)
		dl_wait.sz += len(buf)
		if len(buf) ==0:
			f.flush()
			Log.msg('Transfer terminated by EOD')
			return None
		return Informer.fszp(dl_wait.sz)

	dl_wait.sz = 0;
	Menu.PfmLongtime( Config.p('_compositor').get_layer('dialog'), dl_wait, quiet=quiet ).loop()

	return True

def parse_menu( datafile ):
	res = []
	datafile.seek(0)
	tail = ''
	for portion in datafile.readlines():
		portion = tail + portion
		try:
			(portion , tail) = portion.split("\r\n",1)
		except ValueError:
			entry =  {
				'type': 'gopher',
				'text': 'mailformed line (1)',
				'gtype': '3',
				'host' : '',
				'port' : '0',
				'xtype': None,
			}

		entry = {
			'type': 'gopher',
			'text': 'gopher entry',
			'gtype': 'i',
			'host' : '',
			'port' : '0',
			'xtype': None,
		}

		try:
			entry['gtype'] = portion[:1]
			(
				entry['text'],
				entry['path'],
				entry['host'],
				entry['port'],
			) = portion[1:].split("\t",3)
			entry['text'] = unicode(entry['text'],'utf-8')
		except ValueError:
			entry =  {
				'type': 'gopher',
				'text': 'mailformed line (2)',
				'gtype': '3',
				'host' : '',
				'port' : '0',
				'xtype': None,
			}

		if "\t" in entry['port']:
			(entry['port'],entry['xtype']) = entry['port'].split("\t",1)

		try:
			entry['port'] = int(entry['port'])
		except ValueError:
			entry['port'] = 0

		res.append( entry )

	return res


def get_gopherdir( host, path='/', port=70 ):
	f = StringIO.StringIO()
	download_data( f, host, path, port )
	menu = 	parse_menu( f )
	f.close()
	del f
	return menu
