
import pygame, time, os

from pfm import ModLoad, _threading

Config     = ModLoad("core", "Config")
EntryTypes = ModLoad("core", "EntryTypes")
Lazy	   = ModLoad("core", "Lazy")
Log		   = ModLoad("core", "Log")

MODE_NONE  = 0
MODE_BRIEF = 1
MODE_FULL  = 2

__globalinformer = None

def __sti():
	global __globalinformer
	if __globalinformer == None:
		__globalinformer = PfmInformer()
	return __globalinformer

def set_text(text):
	__sti().set_text(text)

def tick(text):
	"""Informer tick"""
	pass

class PfmInformer:
	"""Status UI layer"""
	def __init__(self):
		self.has_start = False
		self.text = "INFORMER"
		self.layer = Config.p('_compositor').get_layer('status')
		self.set_startpoint (0,0)

	def set_startpoint(self, x, y):
		self.has_start = True
		self.s_x = x
		self.s_y = y

	def set_text(self, text):
		if text != self.text:
			print "Informer update"
			self.text = text
			self.render( self.layer.start() )
			self.layer.stop()

	def render(self, surf):
		pad = Config.p('padding')

		fnt = Config.p('_font')
		tmp = fnt.render( self.text, True, Config.p('textcolor') )

		w = tmp.get_width() + pad*2
		h = tmp.get_height() + pad*2

		x = surf.get_width() - pad - w
		y = 0

		surf.blit( tmp, (x+pad,y+pad, tmp.get_width(), tmp.get_height() ) )
		pygame.draw.rect(surf, Config.p('linecolor'), surf.get_rect(), 1)

def fszp(size):
	"""filesize cuteprint"""
	kmg = ['b','Kb','Mb','Gb','Tb','Pb']
	div = 1024.0
	idx = 0
	p = float(size)
	while p > div:
		p /= div
		idx += 1
	if idx >= len( kmg ):
		idx = len( kmg ) - 1
	if idx == 0:
		return "%d %s" % ( size, kmg[idx] )
	else:
		return "%.2f %s" % ( p, kmg[idx] )


class PfmQuickView:
	"""Qickview"""
	def __init__(self):
		self._layer = Config.p('_compositor').get_layer('hints')
		self._entry = None
		self._mode = MODE_NONE
		#self._proc = None
		self._prevpw = None

		# content bitmap
		self._pane = None
		self._text = []
		self._renderlk = _threading.Lock()
	
	def pw_maxarea(self):
		return (self._layer.get_width() * 1/2, self._layer.get_height() * 3/4)

	def mode(self, new_mode=None):
		if new_mode!=None:
			self._mode = new_mode
		if self._mode == MODE_NONE:
			self._layer.set_visible( False )
		else:
			self.refresh_data()

		return self._mode

	def next_mode(self):
		global MODE_NONE, MODE_BRIEF, MODE_FULL
		self._mode = {
			MODE_NONE  : MODE_BRIEF,
			MODE_BRIEF : MODE_FULL,
			MODE_FULL  : MODE_NONE,
		}[self._mode]

		if self._mode == MODE_NONE:
			self._layer.set_visible( False )
		else:
			self.refresh_data()

	def set_entry( self, entry ):
		if self._prevpw:
			self._prevpw.stop()
			self._prevpw = None

		self._entry = entry
		print "ENTRY SET"
		self.refresh_data()
		print "DATA REFRESHED"

	def refresh_data(self):
		global MODE_NONE, MODE_BRIEF, MODE_FULL

		print "RFR", self._mode
		if self._mode in [MODE_BRIEF]:
			print "mode 1"
			self.fill_shortinfo()

		if self._mode in [MODE_FULL]:
			print "mode 2"
			self.fill_extinfo()

		self.render_texts()
		self.render()
		print "RFR END"

	def _pane_resize( self, neww, newh, clear=False ):
		#self._renderlk.acquire()
		if clear:
			if self._pane != None:
				del self._pane
				self._pane = None
			old = False
		else:
			old = self._pane

		self._pane = pygame.Surface( ( neww, newh ) , pygame.SRCALPHA )

		if old:
			self._pane.blit(old,(0,0))
		#self._renderlk.release()

	def render_texts( self, exttext=None ):
		print "render texts>>>"
		self._renderlk.acquire()

		tx = list(self._text)
		if exttext:
			tx.append(exttext)

		y = 0
		pad = Config.p('padding')
		col = Config.p('textcolor-h2')
		mw = mh = 0

		# apprise pane size
		for v in tx:
			text = EntryTypes.astext_render( v, col )
			y += text.get_height()
			if mw < text.get_width(): mw = text.get_width()

		mh = y

		self._pane_resize( mw+pad*2, mh+pad*2, clear=True )

		y = 0
		for v in tx:
			text = EntryTypes.astext_render( v, col )
			self._pane.blit( text, (  pad, y ) )
			y += text.get_height()

		self._renderlk.release()

	def append_image(self, img):
		self._renderlk.acquire()
		w = img.get_width()
		h = img.get_height()
		y = 0
		if self._pane:
			y = self._pane.get_height()
			h = h + y
			if w < self._pane.get_width():
				w = self._pane.get_width()

		self._pane_resize( w, h , clear=False )
		self._pane.blit( img, (  (w-img.get_width())/2, y-1 ) )

		self._renderlk.release()

	def fill_extinfo(self):
		if self._entry:
			self._text = []
			## search for preview plugin...
			for p in self._entry['plugins']:
				if hasattr(p, 'quickView'):
					if self._prevpw:
						self._prevpw.stop()
						self._prevpw = None
					self._prevpw = p.quickView(self._entry, self)
					return
			self._text.append('No qick view');
			self._text.append('for this file');


	def fill_shortinfo(self):
		if self._entry:
			self._text = []
			if self._entry['type'] == 'dirent':
				if self._entry['ftype'] == 'file':
					self._text.append( 'File, '+fszp(self._entry['size']) )
##					self._text.append( str(self._entry['size']) )
					self._text.append( time.strftime('%d.%m.%Y %H:%M:%S', time.localtime(self._entry['ctime']) ) )
					self._text.append( time.strftime('%d.%m.%Y %H:%M:%S', time.localtime(self._entry['mtime']) ) )

				elif self._entry['ftype'] == 'dir':
					self._text.append('Directory')
					self._text.append( time.strftime('%d.%m.%Y %H:%M:%S', time.localtime(self._entry['ctime'])) )
					self._text.append( time.strftime('%d.%m.%Y %H:%M:%S', time.localtime(self._entry['mtime'])) )

				elif self._entry['ftype'] == 'error':
					self._text.append('Error')
					
				for p in self._entry['plugins']:
					self._text.append(p.__class__.name)
		

	def render(self):
		print "QuickView redraw"
		if self._mode == MODE_NONE:
			return

		self._renderlk.acquire()
		surf = self._layer.start()
		if self._pane:
			r = self._pane.get_rect()
			r[0], r[1] = surf.get_width()-self._pane.get_width(), 0
			pygame.draw.rect(surf, (0,0,0), r)
			surf.blit( self._pane, r )
			pygame.draw.rect(surf, Config.p('linecolor'), r, 1)
		else:
			Log.msg("WARN: no pane")
		self._layer.stop()
		self._layer.set_visible(True)
		self._renderlk.release()



class PfmInfoline:
	"""Status UI layer"""
	def __init__(self):
		self.stats = {}
		self.shift = 0
		self.map = False

	def update_info(self, key, val):
		self.stats[key] = val
		self.map = False

	def prerender(self, surf):
		ww = surf.get_width()
		hh = surf.getheight()
		lh = Config.p('_lineheight')

		self.map = pygame.Surface( (ww, lh * len(self.stats) ), pygame.SRCALPHA )

		y = 0
		for k in self.stats.keys():
			sf = EntryTypes.astext_render( text )
			pyagme.draw.rect( sf, Config.p(), sf.get_rect(), 1 )
			self.map.blit( sf, (0, y, sf.get_width(), lh) )
			y += lh


	def render(self, surf):

		if not self.map:
			self.prerender(surf)

		surf.blit( self.map, (0, 0, self.map.get_width(), Config.p('_lineheight') ) )
		pygame.draw.rect(surf, Config.p('linecolor'), (0, 0, self.map.get_width(), Config.p('_lineheight') ), 1)
