
import pygame

from pfm import ModLoad

##import Config, EntryList, InputAbst, EntryTypes
Config     = ModLoad("core", "Config")
EntryList  = ModLoad("core", "EntryList")
EntryTypes = ModLoad("core", "EntryTypes")
InputAbst  = ModLoad("core", "InputAbst")
Lazy       = ModLoad("core", "Lazy")

def Menu( entrylist, initial=False ):
	mnu = PfmMenu( entrylist, Config.p('_compositor').get_layer('dialog'), Config.p('_im'), initial )
	return mnu.loop()

def Dialog( text, anwslist ):
	mnu = PfmDialog ( text, anwslist, Config.p('_compositor').get_layer('dialog'), Config.p('_im') )
	return mnu.loop()

def Infobox( text ):
	PfmDialog (
		text,
		[ { 'type':'menu', 'text':'OK' } ],
		Config.p('_compositor').get_layer('dialog'),
		Config.p('_im')
		).loop()

def Wait( callback, title="Wait" ):
		t = Lazy.Do(callback)

		def _cb(thread=t, title=title):
			if t.isAlive():
				return title
			else:
				return None

		t.start()
		PfmLongtime(Config.p('_compositor').get_layer('dialog'), _cb ).loop()

class PfmMenu:
	""" Modal menu """
	def __init__( self, entrylist, layer, inputManager, initial=False ):
		self.im = inputManager
		self.layer = layer
		self.el = EntryList.PfmEntryList( entrylist, initial )
		# menu width & height
		self.mw = self.el.get_maxwidth()
		self.mh = Config.p('_lineheight') * len(entrylist)

		if self.mh > layer.get_height():
			self.mh = layer.get_height()

		self.x = (layer.get_width()  - self.mw) / 2
		self.y = (layer.get_height() - self.mh) / 2
		self.el.set_xy( self.x, self.y )

	def redraw(self):
		print "Dialog redraw"
		pad   = Config.p('padding')
		brect = ( self.x-pad, self.y-pad, self.mw+pad*2, self.mh+pad*2 )

		surf = self.layer.start()

		surf .blit( Config.p('_BGM').subsurface(brect), brect )
		pygame.draw.rect(surf , Config.p('linecolor'), brect, 1)
		self.el.render(surf)

		self.layer.stop()

	def loop(self):
		binds = Config.p('_keybindings')
		#if not len(self.el.item()): return None
		res   = None
		bflag = True
		self.redraw()
		while bflag:
			for event in self.im.spool( ) :
				if binds.is_act( event, 'KEY_CANCEL' ) :
					bflag = False
					break

				if binds.is_act( event, 'UP' ):
					self.el.prev()
					self.redraw()

				if binds.is_act( event, 'DOWN' ):
					self.el.next()
					self.redraw()

				if binds.is_act( event, 'KEY_OK' ):
					res = self.el.item()
					bflag = False
					break

		self.layer.clear()
		return res

class PfmDialog:
	""" Modal menu """
	def __init__( self, text, entrylist, layer, inputManager ):
		self.im = inputManager
		self.layer = layer
		pad   = Config.p('padding')

		lh = Config.p('_lineheight')

		# the longest word
		self.mw = layer.get_width()/2
		words = []

		for w in text.split(' '):
			i = EntryTypes.astext_render( w+' ')
			if i.get_width() > self.mw :
				self.mw = i.get_width()
			words.append( i )

		# anwsers line's lenght
		self.anws = entrylist
		elw = 0
		for w in entrylist:
			elw += EntryTypes.Entry_render(w).get_width()

		# optimal width
		if elw > self.mw :
			self.mw = elw

		# anwsers padding
		if len(self.anws) <2 :
			self.anws_shift = (self.mw - elw) / 2
			self.anws_pad   = 0
		else:
			self.anws_pad   = (self.mw - elw) / (len(self.anws) - 1)
			self.anws_shift = 0

		# distribute words to lines
		self.lines = []
		line = []
		linew = 0
		for w in words:
			if linew + w.get_width() > self.mw:
				self.lines.append( line )
				line = [ w ]
				linew = w.get_width()
			else:
				line.append( w )
				linew += w.get_width()
		if(len(line)):
			self.lines.append( line )

		self.mh	= lh * ( len(self.lines) + 2 )

		self.botpane = pygame.Surface( ( self.mw, lh ), pygame.SRCALPHA )
		self.toppane = pygame.Surface( ( self.mw, lh * (len(self.lines)+1) ), pygame.SRCALPHA )
		x = y = 0
		for l in self.lines:
			for w in l:
				self.toppane.blit( w, (x, y, w.get_width(), lh ))
				x += w.get_width()
			y += lh
			x = 0

		if self.mw + 2*pad > layer.get_width() : self.mw = layer.get_width() - 2 * pad
		if self.mh + 2*pad > layer.get_height(): self.mh = layer.get_height() - 2 * pad

		self.x = (layer.get_width()  - self.mw) / 2
		self.y = (layer.get_height() - self.mh) / 2
		self.selected = -1

	def redraw(self):
		print 'Dialog redraw'
		pad   = Config.p('padding')

		surf = self.layer.start()

		self.botpane = pygame.Surface( ( self.mw,  Config.p('_lineheight') ), pygame.SRCALPHA )
		i = 0
		x = self.anws_shift
		for e in self.anws:
			sf = EntryTypes.Entry_render( e, None, i==self.selected )
			self.botpane.blit( sf,
					( x, 0,
					sf.get_width(),
					sf.get_height(),
					)
				)
			x += sf.get_width() + self.anws_pad
			i += 1

		brect = ( self.x-pad, self.y-pad, self.mw+pad*2, self.mh+pad*2 )

##			surf .blit( self.originalstate, ( 0, 0, self.surface.get_width(), self.surface.get_height() ) )
		surf.blit( Config.p('_BGM').subsurface(brect), brect )
		pygame.draw.rect ( surf, Config.p('linecolor'), brect, 1)
		surf.blit( self.toppane, (self.x, self.y, self.toppane.get_width(), self.toppane.get_height()))
		surf.blit( self.botpane, (self.x, self.y + self.mh - self.botpane.get_height(), self.botpane.get_width(), self.botpane.get_height()))

		self.layer.stop()


	def loop(self):
		binds = Config.p('_keybindings')
		#if not len(self.el.item()): return None
		res   = None
		bflag = True
		lsel = False

		self.redraw()

		while bflag:
			for event in self.im.spool( ) :
				if binds.is_act( event, 'KEY_OK' ) or binds.is_act( event, 'KEY_CANCEL' ):
					if self.selected >= 0:
						bflag = False
						break

				if binds.is_act( event, 'LEFT' ):
					if self.selected == -1:
						self.selected = 0
					else:
						self.selected -= 1
						if self.selected < 0:
							self.selected = len( self.anws )-1
					self.redraw()

				if binds.is_act( event, 'RIGHT' ):
					if self.selected == -1:
						self.selected = len( self.anws )-1
					else:
						self.selected += 1
						if self.selected >= len( self.anws ):
							self.selected = 0
					self.redraw()

		self.layer.clear()
		return self.anws[self.selected]

import math

class PfmLongtime:
	""" Static informer for a longtime operation"""
	_angle  = 0.0
	def __init__( self, layer, statuscallback, fps=20, quiet=False ):
		self._layer = layer
		self._stat  = statuscallback
		self._fps = fps
		self._quiet = quiet
		self._figure = [(-30,-30),(-30,30),(30,30),(30,-30)]

	def _render(self):
		def rot( vect, th ):
			return ( vect[0]* math.cos(th) - vect[1] * math.sin(th),
			 vect[0] * math.sin(th) + vect[1] * math.cos(th) )
		def sh( vect, r ):
			return ( vect[0] +r[0], vect[1]+r[1] )

		r=(self._layer.get_width()/2, self._layer.get_height()/2)
		transformed = map( lambda v: rot( v, PfmLongtime._angle), self._figure )
		transformed = map( lambda v: sh( v, r ), transformed )

		sf = self._layer.start()
		pygame.draw.polygon(sf, (0,0,0), transformed, 0)
		pygame.draw.polygon(sf, Config.p('linecolor'), transformed, 1)
		sf.blit(self._tx,
			(r[0] - self._tx.get_width()/2, r[1] - self._tx.get_height()/2)
		)
		self._layer.stop()

	def loop(self):
		cl = pygame.time.Clock()
		stat = self._stat()
		if stat != None and not self._quiet:
			self._tx = EntryTypes.astext_render(stat)

		while stat != None :
			PfmLongtime._angle += 0.05
			cl.tick(self._fps)
			xs = stat
			stat = self._stat()
			if stat==None : break
			if xs != stat:
				self._tx = EntryTypes.astext_render(stat)

			if not self._quiet: self._render()

		if not self._quiet: self._layer.clear()


	def hide(self):
		self._layer.clear()

