
import pygame

from pfm import ModLoad

Config     = ModLoad("core", "Config")
EntryTypes = ModLoad("core", "EntryTypes")
Log  = ModLoad("core", "Log")

MODE_SIMPLE = 0
MODE_WRAP   = 1

class PfmTextFile:
	"""Text lister"""
	def __init__(self, file, offset=0):
		Log.msg('PfmTextFile init')
		self._enc	= 'UTF-8'
		
		self._f		= file			# file object
		self._ofs	= offset		# offset inside the file
		self._layer = Config.p('_compositor').get_layer('screen')

		self.fmt_lines = []			# formatted lines
		self.orig_pos  = []			# position of original lines

		self.pos = 0	# logical selected line in file

		self.fmt_width = self._layer.get_width() - Config.p('padding')*4;
		self.font = Config.p('_font')
		self.feedheight = Config.p('_lineheight')

		self._pagebuf_size = 1024 * 5
		self._font = Config.p('_font')

		self.read_words(self._ofs)

	def compact(self):
		cmp = []
		for l in self.fmt_lines:
			fd = 'L) '
			for w in l:
				fd += w[1] + ' '
			cmp.append(fd)
		return cmp


	def get_line_simple(self, pos):
		pos, idx = self.orig_pos[pos]
		return self.fmt_lines[pos][idx][1]

	def get_line_wrapped(self, pos):
		return (' '.join( [ x[1] for x in self.fmt_lines[pos]] ))

	def read_words(self, ofs):
		"""Read window from file and split it to"""
		self._f.seek(ofs)
		self.append_text( self._f.read(self._pagebuf_size))
		return self._f.tell()

	def append_word(self, word):
		def lwidth(line):
			return reduce( lambda x,y: x+y[0], line, 0 )

		if len(self.fmt_lines)==0:
			self.fmt_lines = [ [] ]

		ll = self.fmt_lines[-1]
		lw = lwidth(ll)

		gtext = self.font.size(word+' ')
		if lw+gtext[0] > self.fmt_width:
			## feed line
			ll = [ tuple([gtext[0], word]) ]
			self.fmt_lines.append( ll )
		else:
			## append word
			ll.append( tuple( [gtext[0], word] ) )
			self.fmt_lines[-1] = ll

	def append_line(self, line):
		space = [ ' ', '\t' ]
		line = [line]

		for c in space:
			r = []
			for w in line:
				for ww in w.split(c):
					r.append(ww)
			line = r

		for w in line:
			try:
				self.append_word( w.decode( self._enc ) )
			except UnicodeDecodeError:
				self.append_word( w )

		self.fmt_lines.append( [ ] )

	def append_text(self, text):
		eol   = [ '\r\n', '\n\r', '\n', '\r' ]
		lines = [text]

		for c in eol:
			r = []
			for w in lines:
				for ww in w.split(c):
					r.append(ww)
			lines = r

		for w in lines:
			self.orig_pos.append( len(self.fmt_lines) )
			self.append_line(w)

	def next(self):
		self.pos+=1
		if( self.pos > len(self.fmt_lines)-1 ): self.pos = 0

	def prev(self):
		self.pos-=1
		if( self.pos<0 ): self.pos = len(self.fmt_lines) - 1

	def next_page(self):
		self.pos += self.pagesize
		if( self.pos > len(self.list)-1 ): self.pos = len(self.list)-1

	def prev_page(self):
		self.pos -= self.pagesize
		if( self.pos<0 ): self.pos = 0

	def sh(self, wheight):
		""" Compute new eye-candy vertical shift """
		y_pos = ( self.pos * self.feedheight )
		self.y_sh = y_pos # - wheight/2 # hh - screen height
		if self.y_sh<0: self.y_sh=0

	def get_entry_surface(self, pos):
		return EntryTypes.astext_render( self.get_line_wrapped( pos ) )

	def render(self, surf):
		ww = surf.get_width()
		hh = surf.get_height()
		self.pagesize = hh /  self.feedheight

		self.sh(hh)
		for i in range(0,len(self.fmt_lines)):
			y = i * self.feedheight
			if y - self.y_sh + self.feedheight < 0 : continue
			if y - self.y_sh > hh : return
			su = self.get_entry_surface( i )
			surf.blit( su, ( 0, y-self.y_sh ) )

		return


		for i in range(0,len(self.list)):
			y = i * self.feedheight
			if y - self.y_sh + self.feedheight < 0 : continue
			if y - self.y_sh > hh : return
			su = self.get_entry_surface( i , i==self.pos)
			surf.blit( su, ( self.x, self.y+y - self.y_sh, su.get_width(), su.get_height()) )
