#
# Pythoned File Manager
#
#  Eugene Popov aka EugeneP, 2012
#    mailto:popov_e_n@mail.ru
#    http://uwolke.ru
#    http://wiz.rusbase.net
#
"""Abstract screen mode"""
import pygame, imp, os
from pfm import ModLoad, _threading


Config     = ModLoad("core", "Config")
Menu       = ModLoad("core", "Menu")
Log        = ModLoad("core", "Log")
#SysOps     = ModLoad("core", "SysOps")

Clock = pygame.time.Clock()
modelist = []

def ActivateMode(modename, wait=None):
	global modelist
	"""Save state of current module and activate given.
	You can freely switch from one mode to another.
	This method starts new mode hread and returns immediately.
	Caller should exit on his own.
	"""
	mode  = ModLoad("modes", modename)
	thread = mode.ModeThread()
	thread.setWait(wait)
	modelist.append(thread)
	Log.msg("Mode created")
	thread.start()
	Log.msg("Mode started")
	return thread

def ModalMode(modename, *params):
	"""Starts a modal mode process. Returns after modal mode stops execution.
	Modal mode oobject returned.
	"""
	mode = ModLoad("modes", modename)
	thread = mode.ModeThread()
	thread.modalparams(*params)
	Log.msg( "ModalMode created" )
	Log.msg( "ModalLoop in: ", modename )
	res = thread.loop()
	Log.msg( "ModalLoop out: ", modename )
	return res

#
# forced exit
#
def exit(code):
	exit.code = code

class ModeAbstract(_threading.Thread):

	def __init__(self):
		self._im	= Config.p('_im')
		self._layer	= Config.p('_compositor').get_layer('screen')
		self._modal = False
		self._wait  = False # wait for termination before loop()
		self._modalresult = None
		self._terminated  = False
		_threading.Thread.__init__(self)
		Log.msg( "Mode init: ",self.__module__ )

	def setWait(self, w):
		"""Wait for this thread termination before starting loop"""
		self._wait = w

	def run(self):
		Log.msg( "Loop in: ", self.__module__ )
		
		if self._wait!=None:
			while self._wait.isAlive():
				Clock.tick(10)

		self.load_state()

		self._modalresult = self.loop()

		self.save_state()

		Log.msg( "Loop out: ", self.__module__ )

	### common actions:
	def show_mode_menu(self):
		if self._modal:
			ch = { 'val': 'exit' }
		else:
			m = ModeMenu()
			m.append(
				{
					'type' : 'menu',
					'text' : 'Exit',
					'val'  : 'exit',
				}
				)
			ch = Menu.Menu(m, initial = self.__module__ )

		if ch:
			if ch['val'] == 'exit':
				self.terminate()
				return

			if ch['val'] != self.__module__:
				self.terminate()
				ActivateMode(ch['val'], self)

			return
		return

	#
	# every mode's loop must check isTerminated()
	#
	def isTerminated(self):
		return self._terminated

	def terminate(self):
		Log.msg( "Terminated: ", self.__module__)
		self._terminated = True

	def modalparams(self, *params):
		self._modal = True
		self._modalparams(*params)

	#
	# what loop() has returned
	#
	def modalresult(self):
		return self._modalresult

	def state_file_name(self):
		return os.path.join( Config.p('_userdir'), self.__module__+'.state' )

	def load_state(self):
		data = Config.grow_from_config( {}, self.state_file_name())
		Log.msg("State loaded: ",data)
		self._restore_state(data)

	def save_state(self):
		data = self._get_state()
		Config.save_to_config( data, self.state_file_name() )
		Log.msg("State saved: ",data)

	### abstracts:

	def _modalparams(self, *params):
		pass

	def _get_state(self):
		return {}

	def _restore_state(self, sdata):
		pass

	def render(self):
		pass

	def loop(self):
		pass



def ModeList(mdir="./modes"):
	"""List of all available modes"""
	if not hasattr( ModeList, 'cache' ):
		ModeList.cache = {}
		for fn in os.listdir( mdir ):
			base, ext  = os.path.splitext(fn)
			if( ext =='.py' or ext=='.pyc') and not base in ['Mode','ModeView']:
				ModeList.cache[ base ] = True
		for mn in ModeList.cache.keys():
			module = ModLoad("modes", mn)
			ModeList.cache[ mn ] = module.ModeName
	return ModeList.cache

def ModeMenu( ):
	menu = []
	for mn, title in ModeList().iteritems():
		menu.append( {
			'type':'menu',
			'text':title,
			'val' :mn,
		})

	return menu

def getActiveModes():
	global modelist
	modelist = filter( lambda x: x.isAlive(),modelist)
	return modelist
