#
# Pythoned File Manager
#
#  Eugene Popov aka EugeneP, 2012
#    mailto:popov_e_n@mail.ru
#    http://uwolke.ru
#    http://wiz.rusbase.net
#
import pygame, os, shutil
#import Config, Informer, EntryList, DirectoryNavigator, Stack, InputAbst, Menu
#import ModeTextEd, ModeView, ModeNet, SysOps

from pfm import ModLoad
Config		= ModLoad("core", "Config")
Informer	= ModLoad("core", "Informer")
Menu		= ModLoad("core", "Menu")
Stack		= ModLoad("core", "Stack")
Log			= ModLoad("core", "Log")
SysOps		= ModLoad("core", "SysOps")
EntryList	= ModLoad("core", "EntryList")
DirectoryNavigator	= ModLoad("core", "DirectoryNavigator")
InputAbst	= ModLoad("core", "InputAbst")

Mode =  ModLoad("modes", "Mode")
ModeName = "File Manager"

class ModeThread (Mode.ModeAbstract):
	"""File manager mode"""
	def __init__(self):
		self.dirnav = DirectoryNavigator.PfmDirectoryNavigator()
		self.ts  = EntryList.PfmEntryList( self.dirnav.get_list() )
		self.st  = Stack.PfmEntryStack( )
		self.inf = Informer.PfmQuickView()
		super(ModeThread,self).__init__()

	def render(self):
		Log.msg("FM redraw")
		Informer.set_text(self.dirnav.dir)
		surf = self._layer.start()
##		surf.blit( Config.p('_BGM'), (0,0) )
##		nf.render(surf)
		self.ts.render(surf)
		self.st.render(surf)
		self._layer.stop()

	def _get_state(self):
		fn = ''
		item = self.ts.item()
		if item:
			fn = item['text']

		mode = self.inf.mode();
		## destructor function		
		self.inf.mode(Informer.MODE_NONE)

		return {
			'navigator_dir'		: self.dirnav.dir,
			'navigator_file'	: fn,
			'navigator_qv_mode'	: mode
		}

	def _restore_state(self, sdata):
		parent = None

		if sdata.has_key('navigator_file'):
			parent = sdata['navigator_file']

		if sdata.has_key('navigator_dir'):
			self.dirnav.chdir( sdata['navigator_dir'] )
			self.ts.relist( self.dirnav.get_list(), parent)
		else:
			self.dirnav.chdir()

		if sdata.has_key('navigator_qv_mode'):
			self.inf.mode(sdata['navigator_qv_mode'])

	def do_mkdir(self, fm, item):
		Informer.set_text('New dir')
		text = Mode.ModalMode('ModeTextEd', 'New Dir')
		Log.msg("mkdir ",text)
		if text != None:
			def _mkdir(dir=self.dirnav.dir, text=text):
				try:
					os.mkdir( os.path.join( dir , text ) )
				except Exception, e:
					Menu.Infobox( 'MKDIR error ' + str(e) )

			Menu.Wait(_mkdir, "mkdir")
			self.dirnav.chdir( self.dirnav.dir )
			self.ts.relist( self.dirnav.get_list(), text+'/' )
		self.render()
		
	def do_touch(self, fm, item):
		Informer.set_text('New file')
		text = Mode.ModalMode('ModeTextEd', 'New File')
		if text != None:
			def _touch(dir=self.dirnav.dir, text=text):
				try:
					open( os.path.join( dir, text ),'ab' ).close();
				except Exception, e:
					Menu.Infobox( 'File creationg error ' + str(e) )

			Menu.Wait(_touch, "touch")
			self.dirnav.chdir( self.dirnav.dir )
			self.ts.relist( self.dirnav.get_list(), ed.text )
		self.render()
		
	def do_copy(self, fm, item):
		top =  self.st.top()
		ev = Menu.Dialog(
			'Copy here?',
			[
				{ 'type':'menu', 'text':'Copy', 'val':True },
				{ 'type':'menu', 'text':'Do not', 'val':False },
			]
		)
		if ev['val'] :
			def _copy(dir=self.dirnav.dir, top=top):
				try:
					fn = os.path.basename(top['fullpath'])
					if top['ftype'] == 'dir':
						shutil.copytree( top['fullpath'], os.path.join( dir, fn ) )
					else:
						shutil.copy( top['fullpath'], os.path.join( dir, fn ) )
					self.st.pop()
				except Exception, e:
					Menu.Infobox( 'Copying error ' + str(e) )

			Menu.Wait(_copy, "copy")
			self.dirnav.chdir( self.dirnav.dir )
			if item :
				self.ts.relist( self.dirnav.get_list(), item['text'] )
			else:
				self.ts.relist( self.dirnav.get_list() )
		self.render()
		
		
	def do_move(self, fm, item):
		top =  self.st.top()
		ev = Menu.Dialog(
			'Move here?',
			[
				{ 'type':'menu', 'text':'Move', 'val':True },
				{ 'type':'menu', 'text':'Do not', 'val':False },
			]
		)
		if ev['val'] :
			def _move(dir=self.dirnav.dir, top=top):
				try:
					fn = os.path.basename(top['fullpath'])
					shutil.move(top['fullpath'], os.path.join( dir, fn ) )
					self.st.pop()
				except Exception, e:
					Menu.Infobox( 'Moving error ' + str(e))

			Menu.Wait(_move, "move")
			self.dirnav.chdir( self.dirnav.dir )
			if item :
				self.ts.relist( self.dirnav.get_list(), item['text'] )
			else:
				self.ts.relist( self.dirnav.get_list() )
		
	def entry_menu(self,item):
		emptymenu = [
			{ 'type':'menu', 'text':'New Dir', 'call':self.do_mkdir },
			{ 'type':'menu', 'text':'New File','call':self.do_touch },
		]
		

		stackmenu = [
			{ 'type':'menu', 'text':'Copy from Stack',  'call':self.do_copy },
			{ 'type':'menu', 'text':'Move from Stack',  'call':self.do_move },
		]
		
		mnu = list(emptymenu)
		
		if item.has_key('plugins'):
			for p in item['plugins']:
				mnu += p.menuItems(item)
				
		if self.st.top():
			mnu += stackmenu
		
		e = Menu.Menu( mnu )
		if e and e.has_key('call'):
			mode = self.inf.mode()
			e['call']( self, item );
			self.inf.mode(mode)
			
	
	def do_open(self, item):
		op = []
		#
		# first "open" plugin
		if item.has_key('plugins'):
			for p in item['plugins']:
				if hasattr(p, 'open'):
					op.append(p)	
					
		if len(op)==0 : return
		if len(op)>1 :
			mnu = [ 
				{'type':'menu', 'text':'Open as '+p.name,'call':p.open} 
				for p in op 
				]
				
			e = Menu.Menu( mnu )
			if e and e.has_key('call'):
				mode = self.inf.mode()
				e['call']( self, item );
				self.inf.mode(mode)
						
		else:
			mode = self.inf.mode()
			op[0].open( self, item );
			self.inf.mode(mode)
	
	def loop(self):
		binds = Config.p('_keybindings')
		dir = self.dirnav
		ts  = self.ts
		st  = self.st
		inf = self.inf

		if(ts.item()):
			inf.set_entry(ts.item())
		self.render()

		while not self.isTerminated():
			Mode.Clock.tick(30)
			for event in self._im.spool( ) :
				if  binds.is_act( event, 'MODE_KEY_SYSMENU' ):
					mode = self.inf.mode()
					self.inf.mode(Informer.MODE_NONE)
					self.show_mode_menu()
					self.inf.mode(mode)
					self.render()
					continue

				if binds.is_act( event, 'UP' ):
					ts.prev()
					if ts.item() :
						inf.set_entry(ts.item())
					self.render()

				if binds.is_act( event, 'DOWN' ):
					ts.next()
					if ts.item() :
						inf.set_entry(ts.item())
					self.render()

				if binds.is_act( event, 'FM_PAGEUP' ):
					ts.prev_page()
					if ts.item() :
						inf.set_entry(ts.item())
					self.render()

				if binds.is_act( event, 'FM_PAGEDOWN' ):
					ts.next_page()
					if ts.item() :
						inf.set_entry(ts.item())
					self.render()

				if binds.is_act( event, 'FM_VIEW_MODE' ):
					inf.next_mode()

				if binds.is_act( event, 'FM_ENTER' ):
					item = ts.item()
					if(item): 
						m = inf.mode()
						inf.mode(Informer.MODE_NONE)
						self.inf.mode(Informer.MODE_NONE)						
						self.do_open(item)
						inf.mode(m)
						self.render()
					continue

				if binds.is_act( event, 'FM_PARENT' ):
					parent = dir.name
					dir.chdir( os.path.join( dir.dir , '..' ) )
					ts.relist( dir.get_list(), parent )
					if ts.item() :
						inf.set_entry(ts.item())
					self.render()

				if binds.is_act( event, 'FM_STACK_PUSH' ):
					if(ts.item()):
						st.push(ts.item())
					self.render()

				if binds.is_act( event, 'FM_STACK_CUT' ):
					st.pop()
					self.render()

				if binds.is_act( event, 'FM_STACK_ROLL1' ):
					st.roll(+1)
					self.render()

				if binds.is_act( event, 'FM_STACK_ROLL2' ):
					st.roll(-1)
					self.render()

				if binds.is_act( event, 'MODE_KEY_ENTRYMENU' ):
					item = ts.item()
					top  = st.top()
					self.entry_menu(item)
					self.render()





