#
# Pythoned File Manager
#
#  Eugene Popov aka EugeneP, 2012
#    mailto:popov_e_n@mail.ru
#    http://uwolke.ru
#    http://wiz.rusbase.net
#
import pygame
import os

#import Config, EntryList, Informer, Menu
#import Gopher

from pfm import ModLoad
Config		= ModLoad("core", "Config")
EntryList	= ModLoad("core", "EntryList")
Menu		= ModLoad("core", "Menu")
Gopher		= ModLoad("core", "Gopher")
Informer	= ModLoad("core", "Informer")
Log			= ModLoad("core", "Log")

Mode =  ModLoad("modes", "Mode")
ModeName = 'GPH@Gopher'

class ModeThread (Mode.ModeAbstract):
	"""Gopher browser"""
	def __init__(self):
		self._menu = EntryList.PfmEntryList( [] )
		self.load_start_page()
		self._history	= []
		self._enc		= 'UTF-8'
		self._dl_dir	= os.path.realpath('./downloads')
		super(ModeThread,self).__init__()

	def load_start_page(self):
		_list = [
			{
						'type': 'gopher',
						'text': 'GP2 Gophernet (pfm\'s Home is here)',
						'gtype': '1',
						'host': 'gp2.uwolke.ru',
						'path': '/',
						'port': 70,
			},
			{
						'type': 'gopher',
						'text': 'Floodgap list of servers (gopher://)',
						'gtype': '1',
						'host': 'gopher.floodgap.com',
						'path': '/',
						'port': 70,
			},
			{
						'type': 'gopher',
						'text': 'xkcd mirror (gopher://)',
						'gtype': '1',
						'host': 'gopher.floodgap.com',
						'path': '/fun/xkcd',
						'port': 70,
			},
			{
						'type': 'gopher',
						'text': 'Pygopherd software (gopher://)',
						'gtype': '1',
						'host': 'gopher.quux.org',
						'path': '/devel/gopher/pygopherd',
						'port': 70,
			},
		]
		self._menu.relist( _list )
		Informer.set_text( ModeName )

	def render(self):
		surf = self._layer.start()
		self._menu.render(surf)
		self._layer.stop()

	def redir(self, host, path, port):
		el = Gopher.get_gopherdir(host, path, port);
		if el == None :
			Menu.Infobox( 'Error opening '+host,  self._im );
			return

		Informer.set_text( host + ' - ' + ModeName )

		el.insert( 0,
			{
						'type': 'gopher',
						'text': path+' (here we are)',
						'gtype': 'i',
						'host': '',
						'path': '',
						'port': 0,
			}
		)
		el.append(
			{
						'type': 'gopher',
						'text': host+'/ (this host root)',
						'gtype': '1',
						'host': host,
						'path': '/',
						'port': port,
			}
		)
		self._menu.relist( el )

	def _get_state(self):
		r={
			'encoding'		: self._enc,
			'download_dir'	: self._dl_dir,
		}

		return r

	def _restore_state(self, sdata):
		if sdata.has_key('encoding'): self._enc = sdata['encoding']
		if sdata.has_key('download_dir'): self._dl_dir = sdata['download_dir']

	def add_hist(self,item):
		self._history.insert(0, item )
		if len(self._history)>50:
			self._history = self._history[:-1]

	def loop(self):
		binds = Config.p('_keybindings')
		self.render()

		while not self.isTerminated():
			Mode.Clock.tick(30)
			for event in self._im.spool( ) :
				if  binds.is_act( event, 'MODE_KEY_SYSMENU' ):
					self.show_mode_menu()
					self.render()
					continue

				if binds.is_act( event, 'UP' ):
					self._menu.prev()
					self.render()

				if binds.is_act( event, 'DOWN' ):
					self._menu.next()
					self.render()

				if binds.is_act( event, 'LEFT' ):
					self._menu.prev_page()
					self.render()

				if binds.is_act( event, 'RIGHT' ):
					self._menu.next_page()
					self.render()

				if binds.is_act( event, 'MODE_KEY_MODEMENU' ):
					mx = (
						{ 'type':'menu', 'text':'History',         'val':'history'},
						{ 'type':'menu', 'text':'Open gopher://',  'val':'open'},
						{ 'type':'menu', 'text':'Enc: '+self._enc, 'val':'encoding'},
						{ 'type':'menu', 'text':'Download dir...', 'val':'dldir'},
						{ 'type':'menu', 'text':'Start page',      'val':'start'},
						{ 'type':'menu', 'text':'Exit',            'val':'exit'},
					)

					res = Menu.Menu( mx, self._im )
					if not res:
						self.render()
						continue

					if res['val']=='history':
						self._menu.relist( self._history )
						Informer.set_text('HISTORY - '+ModeName )
						self.render()
						continue

					if res['val']=='start':
						self.load_start_page()
						self.render()
						continue

					if res['val']=='open':
						Informer.set_text('Open url')
						text = Mode.ModalMode('ModeTexEd', 'gopher://' )
						if text!=None :
							t = Gopher.url2entry(text)
							self._history.insert(0, item )
							try:
								self.redir( t['host'], t['path'], t['port'] )
							except:
								Menu.Infobox('Error. Bad URL?')
						self.render()
						continue

					if res['val']=='encoding':
						Informer.set_text('Text encoding')
						enc = Mode.ModalMode('ModeTextEd', self._enc)
						if enc:
							self._enc = enc
						self.render()
						continue

					if res['val']=='dldir':
						Informer.set_text('Download to dir')
						enc = Mode.ModalMode('ModeTextEd', self._dl_dir)
						if enc:
							self._dl_dir = enc
						self.render()
						continue

					if res['val']=='exit':
						return

					self.render()
					continue

				if binds.is_act( event, 'MODE_KEY_ENTRYMENU' ):
					item = self._menu.item()
					try:
						if Gopher.gtypD.has_key(item['gtype']):
							Informer.set_text(Gopher.gtypD[item['gtype']])
						else:
							Informer.set_text( '?type='+item['gtype'] )
					except:
						Informer.set_text('BROKEN ENTRY')

					if item:
						menu = []
						if item['gtype'] in Gopher.grp_type_downloadable:
							menu.append(
							{
								'type':'menu',
								'text':'Download entry',
								'val' :'download',
							}
							)

						if item['gtype'] in Gopher.grp_type_viewastext:
							menu.append(
							{
								'type':'menu',
								'text':'View as text',
								'val' :'download',
							}
							)

						if item['gtype'] in Gopher.grp_type_viewasimage:
							menu.append(
							{
								'type':'menu',
								'text':'View as image',
								'val' :'imag',
							}
							)

						if len(menu):
							e = Menu.Menu(menu) # :)
							if e:
								if e['val'] == 'download':
									download( item, self._dl_dir )

						self.render()
						continue

				if binds.is_act( event, 'KEY_OK' ):
					item = self._menu.item()
					if not item:
						continue

					if item['type'] == 'gopher':

						if item['gtype'] == '1':
							self.add_hist(item)
							self.redir( item['host'], item['path'], item['port'] )
							self.render()
							continue

						if item['gtype'] in Gopher.grp_type_downloadable:
							download( item, self._dl_dir )
							self.render()
							continue


def download(entry, targetdir):
	e = Menu.Dialog('Download file (type='+entry['gtype']+')?',
	[
		{'type':'menu', 'text':'Download', 'val': True  },
		{'type':'menu', 'text':'Don\'t',   'val': False },
	]
	)

	if not e['val']:
		return

	if entry['gtype'] == 'h':
		#
		# hyperlink workaround
		#
		Menu.Infobox('Hyperlinks are not supported yet :(')
		return

	try:
		os.makedirs (targetdir)
	except:
		pass

	fname = os.path.join ( targetdir, entry['text'].strip('/') )
	f = 0
	try:
		f = open(fname, 'wb')
	except:
		Menu.Infobox('Cannot write to target file')
		return
	Log.msg('Downloading to ', fname)

	Informer.set_text('Downloading...')
	Gopher.download_data( f, entry['host'], entry['path'], entry['port'], quiet=False )
	Informer.set_text('Download complete')

	f.close()




